--[[
Name: LibTalentQuery-1.0
Revision: $Rev: 65445 $
Author: Rich Martel (richmartel@gmail.com)
Documentation: http://wowace.com/wiki/LibTalentQuery-1.0
SVN: http://svn.wowace.com/wowace/trunk/LibTalentQuery-1.0
Description: Library to help with querying unit talents
Dependancies: LibStub, CallbackHandler-1.0
License: LGPL v2.1

Example Usage:
	local TalentQuery = LibStub:GetLibrary("LibTalentQuery-1.0")
	TalentQuery.RegisterCallback(self, "TalentQuery_Ready")
	local spec = {}
	...
	TalentQuery:Query(unit)
	...
	function MyAddon:TalentQuery_Ready(e, name, realm)
		local fullName = realm and name .. "-" .. realm or name
		for tab = 1, GetNumTalentTabs(true) do
			local treename, _, pointsspent = GetTalentTabInfo(tab, true)
			if pointsspent > 30 then
				spec[fullName] = treename
			end
		end
		...
	end
]]

local MAJOR, MINOR = "LibTalentQuery-1.0", "$Rev: 65445 $"

local lib = LibStub:NewLibrary(MAJOR, MINOR)
if not lib then return end

local INSPECTDELAY = 1
if not lib.events then
	lib.events = LibStub("CallbackHandler-1.0"):New(lib)
end

local frame = lib.frame
if not frame then
	frame = CreateFrame("Frame", MAJOR .. "_Frame")
	lib.frame = frame
end
frame:UnregisterAllEvents()
frame:RegisterEvent("INSPECT_TALENT_READY")
frame:SetScript("OnEvent", function(this, event, ...)
	return lib[event](lib, ...)
end)

local CheckInspectQueue
local lastUpdateTime = 0
frame:SetScript("OnUpdate", function(this, elapsed)
	lastUpdateTime = lastUpdateTime + elapsed
	if lastUpdateTime > INSPECTDELAY then
		CheckInspectQueue()
		lastUpdateTime = 0
	end
end)
frame:Hide()

local inspectQueue = lib.inspectQueue
if not inspectQueue then
	inspectQueue = {}
	lib.inspectQueue = inspectQueue
end
if next(inspectQueue) then
	frame:Show()
end
local inspectingUnitName = nil

local UnitIsPlayer = _G.UnitIsPlayer
local UnitName = _G.UnitName

local function UnitNameServer(unit)
	local name, server = UnitName(unit)
	if name and server and (server ~= "") then
		name = name .. "-" .. server
	end
	return name
end

local function NameToUnitID(name)
	local prefix, min, max = "raid", -2, GetNumRaidMembers()
	if max == 0 then
		prefix, min, max = "party", -2, GetNumPartyMembers()
	end
	for i = min, max do
		local unit
		if i == -2 then
			unit = "focus"
		elseif i == -1 then
			unit = "mouseover"
		elseif i == 0 then
			unit = "player"
		else
			unit = prefix .. i
		end
		if UnitIsPlayer(unit) and UnitNameServer(unit) == name then
			return unit
		elseif UnitIsPlayer(unit .. "target") and UnitNameServer(unit .. "target") == name then
			return unit
		elseif i >= 0 and UnitIsPlayer(unit .. "pettarget") and UnitNameServer(unit .. "pettarget") == name then
			return unit
		end
	end
	return nil
end

function lib:Query(unit)
	if unit and UnitExists(unit) and UnitIsPlayer(unit) then
		inspectQueue[UnitNameServer(unit)] = true
		frame:Show()
	else
		error("Usage: TalentQuery:Query(unit): 'unit' must be a valid player unit.", 2)
	end
end

function CheckInspectQueue()
	local repeatqueuecheck
	if not inspectingUnitName then
		repeatqueuecheck = true
		for name in pairs(inspectQueue) do
			local unit = NameToUnitID(name)
			if unit then
				if UnitIsVisible(unit) and UnitIsConnected(unit) then
					inspectingUnitName = name
					inspectQueue[inspectingUnitName] = nil
					repeatqueuecheck = false
					NotifyInspect(unit)
					break
				end
			else
				inspectQueue[name] = nil
			end
		end
	else
		repeatqueuecheck = true
	end
	if not repeatqueuecheck then
		frame:Hide()
	end
end

if not lib.NotifyInspect then -- don't hook twice
	hooksecurefunc("NotifyInspect", function(...) return lib:NotifyInspect(...) end)
end
function lib:NotifyInspect(unit)
	-- If this library did not call NotifyInspect, and we are waiting for a query, then
	-- stop waiting, and put the inspectingUnitName back on the queue.
	if inspectingUnitName and inspectingUnitName ~= UnitNameServer(unit) then
		inspectQueue[inspectingUnitName] = true
		inspectingUnitName = nil
	end
end

function lib:INSPECT_TALENT_READY()
	if not inspectingUnitName then
		return
	end
	local name, server = ("-"):split(inspectingUnitName, 2)
	inspectingUnitName = nil
	self.events:Fire("TalentQuery_Ready", name, server)
end
